<?php
declare(strict_types=1);

class GloboManager {
    private $urlSession;
    private $authorization;
    private $vsid;
    private $defaultHeaders;
    private $channels;
    private $qualities;

    public function __construct() {
        require_once __DIR__ . '/../config.php';
        
        $this->urlSession = BASE_URL . 'video-session';
        $this->authorization = AUTH_TOKEN;
        $this->vsid = VSID;
        $this->qualities = QUALITIES;
        
        $this->defaultHeaders = [
            'Content-Type: application/json; charset=UTF-8',
            'Authorization: ' . $this->authorization
        ];

        $this->loadChannels();
    }

    private function loadChannels(): void {
        $jsonContent = file_get_contents(__DIR__ . '/../data/channels.json');
        if ($jsonContent === false) {
            throw new RuntimeException('Erro ao carregar arquivo de canais');
        }
        $this->channels = json_decode($jsonContent, true);
    }

    public function getStreamUrl(string $videoId, string $lat, string $long, string $quality = 'HD'): string {
        $postData = $this->buildPostData($videoId, $lat, $long, $quality);
        $response = $this->makeCurlRequest($postData);
        return $this->getVideoUrl($response);
    }

    public function getDrmKey(string $url): ?string {
        $xmlContent = file_get_contents($url, false);
        if ($xmlContent === false) {
            return null;
        }

        $xml = @simplexml_load_string($xmlContent);
        if ($xml === false) {
            return null;
        }

        $psshElements = $xml->xpath('//cenc:pssh');
        $shortestPsshData = $this->getShortestPssh($psshElements);
        
        return $shortestPsshData ? $this->requestDrmLicense($shortestPsshData) : null;
    }

    private function getShortestPssh($psshElements): ?string {
        if (empty($psshElements)) {
            return null;
        }

        $shortestPsshData = null;
        foreach ($psshElements as $pssh) {
            $psshData = (string)$pssh;
            if ($shortestPsshData === null || strlen($psshData) < strlen($shortestPsshData)) {
                $shortestPsshData = $psshData;
            }
        }
        return $shortestPsshData;
    }

    private function buildPostData(string $videoId, string $lat, string $long, string $quality): array {
        if (!isset($this->qualities[strtoupper($quality)])) {
            throw new InvalidArgumentException('Qualidade inválida');
        }

        $qualitySettings = $this->qualities[strtoupper($quality)];
        
        return [
            "video_id" => $videoId,
            "player_type" => "android_native",
            "player_version" => "36.0.0",
            "quality" => $qualitySettings['quality'],
            "content_protection" => "widevine",
            "tz" => "-03:00",
            "ts" => time(),
            "lat" => $lat,
            "long" => $long,
            "vsid" => $this->vsid,
            "flt" => true
        ];
    }

    private function requestDrmLicense(string $pssh): ?string {
        $ch = curl_init();
        $data = [
            'license' => DRM_LICENSE_URL . '?deviceId=' . DRM_DEVICE_ID,
            'pssh' => $pssh,
            'cache' => 'true',
            'proxy' => '',
            'headers' => 'Referer: https://globoplay.globo.com/',
            'buildInfo' => 'google/sdk_gphone_x86/generic_x86:8.0.0/OSR1.180418.026/6741039:userdebug/dev-keys'
        ];


//print_r($data);

        curl_setopt_array($ch, [
            CURLOPT_URL => DRM_API_URL,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => json_encode($data),
            CURLOPT_HTTPHEADER => [
                'Host: 216.238.117.79:8080',
                'Connection: keep-alive',
                'Accept: application/json, text/plain, */*',
                'Content-Type: application/json'
            ],
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_SSL_VERIFYHOST => false
        ]);

        $response = curl_exec($ch);
        curl_close($ch);

        if ($response === false) {
            return null;
        }

        return $this->processKeyResponse($response);
    }

    private function processKeyResponse(string $response): ?string {
        $data = json_decode($response, true);
        if (!isset($data['keys']) || !is_array($data['keys'])) {
            return null;
        }

        $keys = $data['keys'];
        if (empty($keys)) {
            return null;
        }

        $key = count($keys) >= 4 ? end($keys)['key'] : ($keys[0]['key'] ?? null);
        if (empty($key)) {
            return null;
        }

        $kid = substr($key, 0, 32);
        $keyValue = substr($key, 33);
        
        return $kid . ':' . $keyValue;
    }

    public function getChannels(): array {
        return $this->channels;
    }

    private function makeCurlRequest(array $postData): array {
        $ch = curl_init($this->urlSession);
        if ($ch === false) {
            throw new RuntimeException('Falha ao inicializar CURL');
        }

        curl_setopt_array($ch, [
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => json_encode($postData),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_HTTPHEADER => $this->defaultHeaders,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_TIMEOUT => 30
        ]);
        
        $result = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        if ($result === false) {
            $error = curl_error($ch);
            curl_close($ch);
            throw new RuntimeException("Erro na requisição CURL: $error");
        }

        curl_close($ch);

        if ($httpCode !== 200) {
            throw new RuntimeException("Erro na resposta do servidor: HTTP $httpCode");
        }

        $response = json_decode($result, true);
        if (!is_array($response)) {
            throw new RuntimeException('Resposta inválida do servidor');
        }

        return $response;
    }

    private function getVideoUrl(array $response): string {
        if (!isset($response['sources']) || !is_array($response['sources'])) {
            throw new RuntimeException('Sources não encontrado na resposta');
        }

        if (isset($response['sources'][0]['url'])) {
            return $response['sources'][0]['url'];
        }

        if (isset($response['sources'][1]['url'])) {
            return $response['sources'][1]['url'];
        }

        throw new RuntimeException('URL do vídeo não encontrada');
    }
}